from subprocess import run
from tempfile import TemporaryFile

# import ledger
import pandas as pd


def read_journal_csv(ledger_args):
    names = [
        'Date',
        'Code',
        'Payee',
        'Account',
        'Commodity',
        'Amount',
        'State',
        'Note'
    ]
    with TemporaryFile(mode='w+') as csv_file:
        run(ledger_args, stdout=csv_file, universal_newlines=True)
        csv_file.seek(0)
        journal = pd.read_csv(csv_file, names=names)
    return journal


def categorize(journal):

    j_cat = journal.copy()

    cat = journal['Account'].str.split(':')
    j_cat['0'] = cat.map(lambda c: c[0] if c[0] == 'Budget' else '_')

    cat = cat.map(lambda c: c[1:] if c[0] == 'Budget' else c)
    levels = cat.map(lambda c: len(c)).max()

    for l in range(1, levels+1):
        j_cat[str(l)] = cat.map(lambda c: c[0] if c else '_')
        cat = cat.map(lambda c: c[1:] if c else c)

    return(j_cat)


def reindex_with_datetime(journal):
    j_datetime = journal.assign(Date=pd.to_datetime(journal['Date'])) 
    return j_datetime.set_index('Date')


def is_virtual(account):
    parentheses = (account[0] == '(') and (account[-1] == ')')
    brackets = (account[0] == '[') and (account[-1] == ']')
    return parentheses or brackets


def strip_account_brackets(journal):

    def strip(account):
        return account[1:-1] if is_virtual(account) else account
    
    return journal.assign(Account = journal['Account'].map(strip))


def clean_and_prepare(journal, args):
    clean = reindex_with_datetime(journal)
    if not args.real:
        clean = strip_account_brackets(clean)
    clean = categorize(clean)
    return clean
    

def truncate_last_periods(journal, periods, freq):
    j_sorted = journal.sort_index()
    p_rng = pd.period_range(end=j_sorted.index[-1], periods=periods, freq=freq)
    start = p_rng[0].to_timestamp(how='start')
    return j_sorted[start:], start


def num_levels(journal):
    n = 0
    while str(n) in journal.columns:
        n += 1
    return n


def sum_by_account(journal, keep_date, level=None):
    n = num_levels(journal)
    if level is not None:
        n = min(level+1, n)
    cols = [str(i) for i in range(n)]
    if keep_date:
        cols += ['Date']
    grouped = journal.groupby(cols)['Amount']
    return grouped.sum()


def get_total(journal, level, accounts=[], keep_date=False):
    j_sub = journal.copy()
    for lvl, arg in enumerate(accounts):
        j_sub = j_sub[j_sub[str(lvl)] == arg]
    j_sub = sum_by_account(j_sub, keep_date, level)
    return j_sub


def add_credits_to_expenses(expenses, journal, level, accounts):
    credits = journal[journal['Amount'] > 0.0]
    total_credits = get_total(credits, level, accounts)
    if total_credits.empty:
        return expenses
    else:
        return pd.concat([expenses, total_credits.loc['_']])


def rename_multiindex_accounts(j_tot):

    def get_name(s):
        return ': '.join(s).rstrip(': _')

    cols = j_tot.index.names
    renamed = j_tot.reset_index()
    renamed['Account'] = renamed[cols].apply(get_name, axis=1)
    renamed = renamed.drop(cols, axis=1)
    return renamed.set_index('Account')


def get_commodity(journal):
    return journal['Commodity'].value_counts().index[0]


def group_below_threshold(j_tot, threshold):
    fraction = j_tot / j_tot.sum()
    below = (fraction < threshold)
    if below.squeeze().any():
        below_sum = j_tot[below].sum()
        above = j_tot.drop(j_tot[below].index)
        other = pd.Series(below_sum, index=['Other'])
        return pd.concat([above, other])
    return j_tot
