import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

from .styles import (
    ludget_style,
)
from .munging import (
    add_credits_to_expenses,
    get_commodity,
    get_total,
    group_below_threshold,
    num_levels,
    rename_multiindex_accounts,
    sum_by_account,
    truncate_last_periods,
)


def plot_assets(journal, level=None, periods=365, freq='D'):

    assets = get_total(journal, level, ['_', 'Assets'], keep_date=True)
    assets = assets.loc['_', 'Assets']
    assets = assets.unstack(assets.index.names[:-1])
    # sum instead of asfreq because we might be downsampling depending on freq
    assets = assets.resample(freq).sum()
    assets = assets.cumsum()
    assets, start = truncate_last_periods(assets, periods, freq)
    assets = rename_multiindex_accounts(assets.T).T
    # reverse columns order for nicer plot with cash > checking > savings
    assets = assets.reindex(columns=assets.columns[::-1])
    # remove columns name to declutter the legend
    assets.columns.name = None

    # plotting
    with plt.style.context(['seaborn', ludget_style]):
        fig, ax = plt.subplots(num='assets')
        assets.plot.area(ax=ax, legend='reverse', title='Assets')
        plt.ylabel('Amount ({})'.format(get_commodity(journal)))
        plt.tight_layout()


def plot_budget(journal, level=None, periods=1, freq='M'):

    j_periods, start = truncate_last_periods(journal, periods, freq)

    budget = get_total(journal, level, ['Budget']).loc['Budget']
    budget_pos = (budget >= 0)
    remaining = budget[budget_pos]
    remaining.name = 'Budget'
    overspent = budget[~budget_pos]
    overspent.name = 'Overspent'

    expenses = get_total(j_periods, level, ['_', 'Expenses']).loc['_']
    for accounts in [['_', 'Liabilities'], ['_', 'Assets', 'Savings']]:
        expenses = add_credits_to_expenses(expenses, j_periods, level,
                                           accounts)
    expenses.name = 'Expenses'

    budget = pd.concat([expenses, remaining, overspent], axis=1)
    budget = rename_multiindex_accounts(budget)

    # manipulations to get overspent stacked on top of expenses
    budget['Expenses'] = budget['Expenses'].add(budget['Overspent'],
                                                fill_value=0)
    budget['Overspent'] = budget['Overspent'] * -1

    # prevents Budget:Available expenses from being negative if overspent
    budget.loc[budget['Expenses'] < 0.0, 'Expenses'] = 0.0

    # plotting
    with plt.style.context(['seaborn', ludget_style]):
        fig, ax = plt.subplots(num='budget')
        dates = ' - '.join([str(start.date()), str(j_periods.index[-1].date())])
        title = 'Budget\n{}'.format(dates)
        budget.plot.barh(ax=ax, stacked=True, title=title)
        plt.xlabel('Amount ({})'.format(get_commodity(j_periods)))
        plt.tight_layout()


def plot_cash_flow(journal, periods=12, freq='M'):

    j_periods, start = truncate_last_periods(journal, periods, freq)

    total = get_total(j_periods, 1, ['_'], keep_date=True)
    total = total.loc['_'].loc[['Expenses', 'Income']]
    total = total.unstack(total.index.names[:-1])
    total = total.resample(freq, kind='period').sum()
    total.columns.name = None

    cash_flow = total['Expenses'].add(total['Income'])
    average = cash_flow.mean()

    # plotting
    with plt.style.context(['seaborn', ludget_style]):
        fig, ax = plt.subplots(num='cash_flow')
        total.plot.bar(ax=ax, stacked=True, title='Cash flow')
        plt.axhline(0, color='white', linestyle='-', linewidth=1)
        cash_flow.plot.line(ax=ax, use_index=False, label='Cash flow',
                            legend=True, style='C2o-')
        # hardcode the color instead of using 'C2' to avoid the following bug
        # https://github.com/matplotlib/matplotlib/issues/11376
        plt.axhline(average, label='Average',
                    color='#C44E52', linestyle='--')
        # manually set the order of legend labels
        handles, labels = ax.get_legend_handles_labels()
        handles = [handles[2], handles[3], handles[0], handles[1]]
        labels = [labels[2], labels[3], labels[0], labels[1]]
        plt.legend(handles, labels)
        plt.ylabel('Amount ({})'.format(get_commodity(j_periods)))
        plt.tight_layout()


def plot_expenses(journal, level=None, periods=1, freq='M', threshold=0.05):

    j_periods, start = truncate_last_periods(journal, periods, freq)

    expenses = get_total(j_periods, level, accounts=['_', 'Expenses'])
    expenses = expenses.loc['_', 'Expenses']
    expenses = rename_multiindex_accounts(expenses)
    expenses = expenses.squeeze()
    expenses = group_below_threshold(expenses, threshold)

    # plotting
    with plt.style.context(['seaborn', ludget_style]):
        fig, ax = plt.subplots(num='expenses')
        label = 'Total ({})\n\n{:.2f}'.format(get_commodity(j_periods),
                                              expenses.sum())
        dates = ' - '.join([str(start.date()), str(j_periods.index[-1].date())])
        title = 'Expenses\n{}'.format(dates)
        width = 0.4
        kwds = {
            'autopct': '%.1f%%',
            'pctdistance': (2.0 - width) / 2.0,
            'wedgeprops': dict(width = width),
        }
        expenses.plot.pie(ax=ax, title=title, label='', colormap='summer', **kwds)
        plt.text(0.5, 0.5, label, size='large', horizontalalignment='center',
                 verticalalignment='center', transform=ax.transAxes)
        ax.set_aspect('equal')


def plot_net_worth(journal, periods=12, freq='M'):

    j_periods, start = truncate_last_periods(journal, periods, freq)

    total = get_total(j_periods, 1, ['_'], keep_date=True).loc['_']
    total = total.unstack(total.index.names[:-1])
    if 'Liabilities' not in total.columns:
        total['Liabilities'] = np.nan
    total = total[['Assets', 'Liabilities']]
    total = total.resample(freq, kind='period').sum()
    total = total.cumsum()
    total.columns.name = None

    net_worth = total['Assets'].add(total['Liabilities'])

    # plotting
    with plt.style.context(['seaborn', ludget_style]):
        fig, ax = plt.subplots(num='net_worth')
        total.plot.bar(ax=ax, stacked=True, title='Net worth')
        plt.axhline(0, color='white', linestyle='-', linewidth=1)
        net_worth.plot.line(ax=ax, use_index=False, label='Net worth',
                            legend=True, style='C2o-')
        plt.ylabel('Amount ({})'.format(get_commodity(j_periods)))
        plt.tight_layout()
